#####
###  PHParser - A Gateway to PHP
##   Copyright (C) 2003	Wei He (hewei@ied.org.cn)
#    Copyright (C) 2002	Maik Jablonski (maik.jablonski@uni-bielefeld.de)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

# Path to the PHP-CGI-Interpreter

PHPath="/var/www/cgi-bin/php"
# PHPath="/usr/local/bin/php"   # Linux & other *ix
# PHPath="c:\php\php"           # Windows

PHPEOL   = "\r?\n"    

from Globals import HTML, DTMLFile, InitializeClass
from OFS.content_types import guess_content_type
from OFS.DTMLMethod import decapitate
from OFS.DTMLDocument import DTMLDocument
from AccessControl import getSecurityManager
from ZPublisher.HTTPRequest import FileUpload
from urllib import quote
from os import getenv,putenv,popen,popen2,unlink,environ
from urllib import urlencode
import cgi, re, string, tempfile

_marker=[]

default_dd_html="""
<dtml-var standard_html_header>

<?php
	echo "<h2><dtml-var title_or_id> / {$ZOPE_VARS['BASE0']}$PHP_SELF</h2>";
	echo "<h2>PHParser extended global variables: </h2><pre>\$ZOPE_VARS=";
	print_r($ZOPE_VARS);
	echo "</pre>";
	phpinfo();
?>

<dtml-var standard_html_footer>
"""

addPHPForm = DTMLFile('dtml/addPHPForm', globals())

# This code is taken and slightly modified from OFS/DTMLDocument.py

def addPHParser(self, id, title='', file='', REQUEST=None, submit=None):
    """Add a PHParser object with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=default_dd_html
    id=str(id)
    title=str(title)
    ob=PHParser(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''


class PHParser(DTMLDocument):
    """ PHParser: It's a DTMLDocument which will be parsed by PHP after
        rendering the DTML """    

    meta_type = 'PHParser'
    manage_options=DTMLDocument.manage_options

    # Transform Python data value into PHP

    def transformValue(self, value):

        if type(value) in (type(0), type(0.0)):
            #
            # number
            #
            result = str(value)

        elif type(value) in (type([]), type(())):
            #
            # list or turple
            #
            result = "array("
            if len(value) > 0:
                for v in value:
                    result += self.transformValue(v) + ","
                result = result[:-1]
            result += ")"

        elif type(value) == type({}):
            #
            # dict
            #
            result = "array("
            if len(value) > 0:
                for (k,v) in value.items():
                    result += "'" + str(k) + "'=>" + self.transformValue(v) + ","
                result = result[:-1]
            result += ")"

        else:
            #
            # string and others
            #
            result = "'" + string.replace(str(value), "'", "\\'") + "'"

        return result

    # This is the function where the document is piped through PHP

    def __phparse__(self,src,REQUEST={}):
        """ Pipes the rendered DTML through PHP """

        varfix = ""
        """ Force REQUEST to generate all items. Any better idea? """
        REQUEST.text()

        """ Construct $ZOPE_VARS from REQUEST.other """
        zopevars = {}
        for item in REQUEST.other.keys():
            if not item in ('RESPONSE', 'PUBLISHED', 'PARENTS'):
                 zopevars.update({item: REQUEST.other[item]})
        varfix += "$ZOPE_VARS=" + self.transformValue(zopevars) + "; "

        """ Fix PATH_INFO variable """
        varfix += "$HTTP_SERVER_VARS['PATH_INFO'] = $_SERVER['PATH_INFO'] = substr($_SERVER['PHP_SELF'], strlen($_SERVER['REDIRECT_SCRIPT_URL'])); "
        varfix += "if (isset($GLOBALS['PATH_INFO'])) { $GLOBALS['PATH_INFO'] = $_SERVER['PATH_INFO'];} "

        # In case you want to prepend your own codes.
        #varfix += "include('" + INSTANCE_HOME + "/php.inc'); "

        # we create the shell script on disk
        temp = tempfile.mktemp()
        ftemp = open(temp, "w")
        ftemp.write('<?php ' + varfix + ' ?>')
        ftemp.write(src)
        ftemp.close()
       
        cgienv={} 
 
	for (k,v) in REQUEST.environ.items():
            cgienv[k]=str(v)

        # todo: Virtual host support 
        cgienv['SCRIPT_NAME']='/' + self.absolute_url(1)
        cgienv['PHP_SELF']='/' + self.absolute_url(1) 

        # Required by PHP CGI. 
        cgienv['SCRIPT_FILENAME']=temp
        cgienv['PATH_TRANSLATED']=temp

        cgienv['REDIRECT_STATUS']='200'
        cgienv['REDIRECT_SCRIPT_URL']=cgienv['SCRIPT_NAME']
        cgienv['REDIRECT_SCRIPT_URI']=REQUEST['URL0']

        # Emulate REQUEST_URI
        cgienv['REQUEST_URI']=string.replace(REQUEST['URL0'], REQUEST['BASE0'], '')
        if (cgienv['PATH_INFO'][-1] == '/'):
           cgienv['REQUEST_URI']+='/'
        if (cgienv.has_key('QUERY_STRING')):
           cgienv['REQUEST_URI']+='?' + cgienv['QUERY_STRING']

	# Ladies and gentlemen... and now... PHParsing...

        cmd='/bin/env -i'
        for (k,v) in cgienv.items():
           cmd+=' ' + k + '="' + v + '"'
        
        (stin,stout)=popen2(cmd + ' ' + PHPath)

        """ Reconstruct the form data (fields and files) """
        # 
        # Thanks to
        # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/146306
        #
        if string.upper(cgienv['REQUEST_METHOD']) == 'POST':
            multipart = re.search("^multipart/form-data; boundary=(.*)", cgienv['CONTENT_TYPE'])
            if (multipart):
               #
               # form data is posted as multipart/form-data
               #
               BOUNDARY=multipart.groups()[0]
               CRLF = '\r\n'
               L = []
               for (k,v) in REQUEST.form.items():
                  L.append('--' + BOUNDARY)
                  if isinstance(v, FileUpload):
                     #
                     # handle file uploads
                     #
                     L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (k, v.filename))
                     L.append('Content-Type: %s' % v.headers['content-type'])
                     content=v.read()
                  else:
                     #
                     # handle input fields
                     #
                     L.append('Content-Disposition: form-data; name="%s"' % k)
                     content=v
                  L.append('')
                  L.append(content)
               L.append('--' + BOUNDARY + '--')
               L.append('')
               formdata = CRLF.join(L)
            else:
               #
               # form data is posted as application/x-url-encoded
               #
               formdata = urlencode(REQUEST.form, 1)

            cgienv['CONTENT_LENGTH'] = str(len(formdata))
            stin.write(formdata)
        stin.flush()
        stin.close() 
        
        raw=stout.read()
        stout.close()
        unlink(temp)

	# Now decode the Header and Body of the result
        phpresult=re.search("(?s)(?P<header>^.*?)%s(?P<body>.*)"%(PHPEOL*2),raw)
        if phpresult:
            (phpHeader,phpBody) = phpresult.groups() 
            for line in re.split(PHPEOL, phpHeader):
                if ':' in line:
                    (name,value) = line.split(':',1)
                    if name.lower()=='location':
                        REQUEST.RESPONSE.redirect(value.strip())
                    else:
                        REQUEST.RESPONSE.setHeader(name,value.strip())
        else:
            REQUEST.RESPONSE.setHeader("Content-type", "text/plain")
            phpBody = ''
        return(phpBody)

    # This is the original code from OFS/DTMLDocument.py
    # Only a call to __phparse__ is added (see below)

    def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        """Render the document given a client object, REQUEST mapping,
        Response, and key word arguments."""

        if not self._cache_namespace_keys:
            data = self.ZCacheable_get(default=_marker)
            if data is not _marker:
                # Return cached results.
                return data

        kw['document_id']=self.getId()
        kw['document_title']=self.title
        if hasattr(self, 'aq_explicit'):
            bself=self.aq_explicit
        else: bself=self
        
        security=getSecurityManager()
        security.addContext(self)

        try:
            if client is None:
                # Called as subtemplate, so don't need error propigation!
                r=apply(HTML.__call__, (self, bself, REQUEST), kw)
                r=self.__phparse__(r,bself.REQUEST)
                if RESPONSE is None: result = r
                else: result = decapitate(r, RESPONSE)
                if not self._cache_namespace_keys:
                    self.ZCacheable_set(result)
                return result

            r=apply(HTML.__call__, (self, (client, bself), REQUEST), kw)
            r=self.__phparse__(r,bself.REQUEST)
	    
            if type(r) is not type('') or RESPONSE is None:
                if not self._cache_namespace_keys:
                    self.ZCacheable_set(r)
                return r

        finally: security.removeContext(self)
	
        have_key=RESPONSE.headers.has_key
        if not (have_key('content-type') or have_key('Content-Type')):
            if self.__dict__.has_key('content_type'):
                c=self.content_type
            else:
                c, e=guess_content_type(self.__realname__, r)
            RESPONSE.setHeader('Content-Type', c)
        result = decapitate(r, RESPONSE)
        if not self._cache_namespace_keys:
            self.ZCacheable_set(result)
        return result

InitializeClass(PHParser)
